<div align="center"><img height="180" src="https://gitlab.iscpif.fr/gargantext/main/raw/master/images/logo.png"></div>

&nbsp;
# Gargantext with Haskell (Backend instance)

![Haskell](https://img.shields.io/badge/Code-Haskell-informational?style=flat&logo=haskell&color=6144b3)&nbsp;&nbsp;![Nix](https://img.shields.io/badge/Package%20manager-Nix-informational?style=flat&logo=nixos&color=6586c8)&nbsp;&nbsp;![Cabal](https://img.shields.io/badge/Tools-Cabal-informational?style=flat&logo=cabal&color=567dd9)&nbsp;&nbsp;![Stack](https://img.shields.io/badge/Tools-Stack-informational?style=flat&logo=stack&color=6144b3)&nbsp;&nbsp;![GHC](https://img.shields.io/badge/Tools-GHC-informational?style=flat&logo=&color=2E677B)&nbsp;&nbsp;![Docker](https://img.shields.io/badge/Tools-Docker-informational?style=flat&logo=docker&color=003f8c)

#### Table of Contents
1. [About the project](#about)
2. [Installation](#install)
3. [Use cases](#use-cases)
4. [Notes for developers](#develop)
5. [`haskell-language-server`](#haskell-language-server)
6. [Running the tests](#running-tests)
7. [Async workers](#async-workers)
8. [Notifications](#notifications)

## About the Project <a name="about"></a>

GarganText explores unstructured texts using NLP, network analysis, and data visualization tools.

Developed by CNRS ISC-PIF, this backend supports the [frontend](https://gitlab.iscpif.fr/gargantext/haskell-gargantext).


## Installation <a name="install"></a>

**Disclaimer:** Project in development. Report issues to improve documentation.

### Prerequisites

Install:
- [GHCup](https://www.haskell.org/ghcup/)
- [Git](https://git-scm.com/book/en/v2/Getting-Started-Installing-Git)
- [Nix](https://nixos.org/download/)
- [Docker Compose](https://docs.docker.com/compose/install/).

Add yourself to the `docker` group (replace `myusername` with your username).

### Getting the source code

```bash
git clone https://gitlab.iscpif.fr/gargantext/haskell-gargantext.git && cd haskell-gargantext
```

### Building Gargantext

#### Building the Backend

Run the install script to build, test, and document:
```bash
./bin/install
```

#### Building the Frontend
This builds GarganText, runs tests, and generates documentation.

Move into the `purescript-gargantext/` directory, then run the install script:
```bash
./bin/install
```
Run the script in `purescript-gargantext/`:
```
cd purescript-gargantext && ./bin/install && cd ..
```

### Initialize
Rename `gargantext-settings.toml_toModify` to `gargantext-settings.toml` and fill fields.

`.gitignore` excludes this file from commits; change passwords safely.

**NOTE** Convert `gargantext.ini` to `gargantext-settings.toml` in a Nix shell:
```shell
n$ ./bin/cli ini
```

### Launching and Initializing the Database

Launch the docker image from inside the dedicated directory:
```bash
cd devops/docker/
docker compose up
```

Initialize the database in `haskell-gargantext/`:
```bash
./bin/cli init # Follow the prompts.
```

Provide master user credentials when prompted.

### Running Gargantext

To run Gargantext, execute from `haskell-gargantext/`:
```bash
./bin/run
```

## Use Cases <a name="use-cases"></a>

### Start Multi-User GUI (Server Mode)
```bash
cabal v2-run gargantext -- server start -m Prod
```


Log in as `user1` / `1resu`.

Start server and all workers:
```bash
cabal v2-run gargantext -- server start-all -m Prod
```

### Command Line Mode tools

#### Compute and Index Cooccurrences
```bash
cabal v2-run gargantext -- filter-terms CorpusFromGarg.csv ListFromGarg.csv Ouput.json
```

### Analyzing the ngrams table repo

Repositories are in `repos` as [CBOR](https://cbor.io/). Decode to JSON and analyze with [jq](https://shapeshed.com/jq-json/):

``` shell
$ cat repos/repo.cbor.v5 | stack exec gargantext-cbor2json | jq .
```
### Documentation

To build documentation, run:

```shell
$ stack build --haddock --no-haddock-deps --fast
```

(in `.stack-work/dist/x86_64-linux-nix/Cabal-3.2.1.0/doc/html/gargantext`).


## Notes for developers <a name="develop"></a>

### Developer Documentation

If you want to contribute to Gargantext, there is [some documentation geared to developers](https://write.frame.gargantext.org/s/805e1ee2bae24079554d24cdbc8ef5ba5c4ef7d83218c1e86c84af8ac269031d).

### Upgrading Haskell Packages

We use `gargantext.cabal`, `cabal.project` and `cabal.project.freeze`
as the source of truth. Ouf ot that, we generate the `stack.yaml` file
for those who prefer to use Stack.

Upgrading packages can be a pain sometimes, with cabal.
``` shell
cat repos/repo.cbor.v5 | stack exec gargantext-cbor2json | jq .
```
### Build Documentation
- Manually remove entries from your `cabal.project.freeze` to make the
Build documentation with:
`stack build --haddock --no-haddock-deps --fast`
  v2-build --constraint tasty==x.y.z.w`
- Generate another `.freeze` with `cabal v2-freeze` once you got the
(Output in `.stack-work/dist/x86_64-linux-nix/Cabal-3.2.1.0/doc/html/gargantext`).


## Developer Notes <a name="develop"></a>

### Contributing
Developer [documentation](https://write.frame.gargantext.org/s/805e1ee2bae24079554d24cdbc8ef5ba5c4ef7d83218c1e86c84af8ac269031d) is available for contributions.
the `.cabal` file versions.
### Upgrade Haskell Packages
We use `gargantext.cabal`, `cabal.project`, `cabal.project.freeze` as the primary sources, generating `stack.yaml` for Stack users.
cabal v2-build --constraint hashable==1.4.3.0
Upgrading packages with Cabal can be complex. Tips:

- Remove entries from `cabal.project.freeze` for flexibility.
- Lock hackage-index in `cabal.project` to avoid newer dependencies.
- Specify build constraints, e.g., `cabal v2-build --constraint tasty==x.y.z.w`.
- Regenerate `.freeze` with `cabal v2-freeze` after successful build.
- Use `cabal gen-bounds` before removing `.freeze`; retry builds.
with `cabal v2-build all`).
Occasionally build with `stack build` to identify stack LTS discrepancies, prompting `.cabal` updates.
### Running Tests
Issues may arise from `allow-newer: *` in `cabal.project`. To resolve, use:
`cabal v2-build --constraint hashable==1.4.3.0`

(This is due to `allow-newer: *`. We prefer strict constraints in `gargantext.cabal`).
Use `stack ls dependencies` to identify compatible packages.
For detailed build info, run:
`cabal v2-build -v servant-server`
or (for Hspec):
Use `-Wunused-packages` with GHC to detect unused packages during `cabal v2-build all`.
See [this discussion](https://discourse.haskell.org/t/whats-your-workflow-to-update-cabal-dependencies/9475) for more tips.
The CI is on gitlab and is controlled by the `.gitlab-ci.yml` file.

### Run Tests
From Nix shell:
`n$ cabal v2-test --test-show-details=streaming`
``` shell
docker build -t cgenie/gargantext:9.4.8 -f ./devops/docker/Dockerfile .
```

Or, outside Nix shell:
```shell
nix-shell --run "cabal v2-test --test-show-details=streaming"
mkdir _build
```
To run specific tests with Tasty:
`cabal v2-test garg-test-tasty --test-show-details=streaming --test-option=--pattern='/job status update and tracking/`

For Hspec:
`cabal v2-test garg-test-hspec --test-show-details=streaming --test-option=--match='/Dispatcher, Central Exchange, WebSockets/'`

1. Ongoing devlopment (on local repo):
   1. In `cabal.project`:
      - add `../hal` to `packages:`
      - turn off (temporarily) the `hal` in `source-repository-package` 
   2. When changes work and tests are OK, commit in repo `hal`
2. When changes are commited / merged:
   1. Get the hash id, and edit `cabal.project` with the **new commit id**
   2. run `./bin/update-project-dependencies`
      - get an error that sha256 don't match, so update the `./bin/update-project-dependencies` with new sha256 hash
      - run again `./bin/update-project-dependencies` (to make sure it's a fixed point now)

> Note: without `stack.yaml` we would have to only fix `cabal.project` -> `source-repository-package` commit id. Sha256 is there to make sure CI reruns the tests.

### GraphQL

Some introspection information.

Playground is located at http://localhost:8008/gql

#### List all GraphQL types in the Playground

```
{
  __schema {
    types {
      name
    }
  }
}
```

#### List details about a type in GraphQL

### Upgrading with Nix <a name="nix-upgrade"></a>
Nix builds from multiple sources. `nix/sources.json` serves as a Sitemap for SBT plugins, Scala/Java libraries, etc., and is loaded by `nix/sources.nix`.
To upgrade dependencies or tools in `nix/sources.json`, run:
```shell
nix-shell --run "niv update"
```
To add a new dependency:
```shell
niv add repo/name
```
Niv maintains `nix/sources.json`, and it can be edited manually.
If issues occur, clean the cache with:
`nix-shell --run "nix-collect-garbage"`
- `packages/gargbase`
- `packages/utility`
https://www.cloudytuts.com/tutorials/docker/how-to-upgrade-postgresql-in-docker-and-kubernetes/
To build only `gargbase`:
`$ cabal v2-build gargbase`

To test changes in `packages/gargbase`, run:
`n$ cabal v2-test testutils --test-show-details=streaming`
$ docker-compose rm postgres
If issues arise, run the full test suite to check other packages:
`n$ cabal v2-test --test-show-details=streaming`
### Troubleshooting
If Haddock or interactive tests cause freezes, try:
`n$ cabal clean`
`n$ cabal configure`

If cached Nix derivations are suspect, clean with:
`n$ nix-collect-garbage`
First save your data:
### Conclusion
You now have guidelines for Gargantext development and use. Contributions for unexplored aspects are welcome!
Upgrade postgresql:
```shell
$ sudo apt install postgresql-server-14 postgresql-client-14
$ sudo apt remove --purge postgresql-13
```
Restore your data:
```shell
$ sudo su postgres
$ psql < gargandb.dump
```

## Conclusion
You now have the instructions to manage Gargantext development and use. Contributions for areas not covered are welcome!

## Running the tests <a name="running-tests"></a>

Running the tests can be done via the following command:

```hs
cabal v2-test --test-show-details=streaming --flags 'test-crypto no-phylo-debug-logs'
```

The flags have the following meaning:

* `test-crypto`: Switch to use very fast (but not production-secure) cryptography, so that tests runs
  faster;
* `no-phylo-debug-logs`: Suppresses the debugging logs which would normally be present in phylo pure (!) code.

In order for some tests to run (like the phylo ones) is **required** to install the `gargantext-cli` via:

```hs
cabal v2-install gargantext:exe:gargantext
```

### Modifying a golden test to accept a new (expected) output

Some tests, like the Phylo one, use golden testing to ensure that the JSON Phylo we generate is
the same as an expected one. This allows us to catch regressions in the serialisation or in the algorithm.
Sometimes, however, we genuinely want to modify the output so that it's the new reference (i.e. the new
golden reference). To do so, it's enough to run the testsuite passing the `--accept` flag, for example:

```hs
cabal v2-test garg-test-tasty --test-show-details=streaming --flags 'test-crypto no-phylo-debug-logs' --test-option=--pattern='/Phylo/' --test-option=--accept"
```

## `haskell-language-server` <a name="haskell-language-server"></a>

If you want to use `haskell-language-server` for GHC 9.4.7, install it
with `ghcup`:
```shell
ghcup compile hls --version 2.7.0.0 --ghc 9.4.7
```

### `haskell-language-server` Installation

If you want to use `haskell-language-server` for GHC 9.4.7, install it with `ghcup`:

```shell
ghcup compile hls --version 2.7.0.0 --ghc 9.4.7
```
https://haskell-language-server.readthedocs.io/en/latest/installation.html
https://haskell-language-server.readthedocs.io/en/latest/installation.html


## Running the tests <a name="running-tests"></a>

Running the tests can be done via the following command:

```hs
cabal v2-test --test-show-details=streaming --flags 'test-crypto no-phylo-debug-logs'
```

The flags have the following meaning:

* `test-crypto`: Switch to use very fast (but not production-secure) cryptography, so that tests runs
  faster;
* `no-phylo-debug-logs`: Suppresses the debugging logs which would normally be present in phylo pure (!) code.

In order for some tests to run (like the phylo ones) is **required** to install the `gargantext-cli` via:

```hs
cabal v2-install gargantext:exe:gargantext
```

### Modifying a golden test to accept a new (expected) output

Some tests, like the Phylo one, use golden testing to ensure that the JSON Phylo we generate is
the same as an expected one. This allows us to catch regressions in the serialisation or in the algorithm.
Sometimes, however, we genuinely want to modify the output so that it's the new reference (i.e. the new
golden reference). To do so, it's enough to run the testsuite passing the `--accept` flag, for example:

```hs
cabal v2-test garg-test-tasty --test-show-details=streaming --flags 'test-crypto no-phylo-debug-logs' --test-option=--pattern='/Phylo/' --test-option=--accept"
```

# Async workers <a name="async-workers"></a>
### Async workers <a name="async-workers"></a>

#### Steps to configure the worker

1. First create our Worker
```bash
createdb gargantext_pgmq
psql -c "ALTER DATABASE \"gargantext_pgmq\" OWNER TO \"gargantua\""
```
2. Then configure you gargantext-settings.toml (see next section)

3. Finally launch the worker:
```bash
nix-shell --run "cabal v2-run gargantext -- worker run --name default"
```

Or launch all worker definitions at once:
```bash
nix-shell --run "cabal v2-run gargantext -- worker run-all"
```

#### Configuration

Edit your `gargantext-settings.toml` file and add this section:
```toml
[worker]

# After this number of seconds, the job will be available again.
# You can set timeout for each job individually and this is the
# preferred method over using defaultVt.
default_visibility_timeout = 1

# default delay before job is visible to the worker
default_delay = 0

[worker.database]
host = "127.0.0.1"
port = 5432
name = "pgmq"
user = "gargantua"
pass = PASSWORD_TO_CHANGE

[[worker.definitions]]
name = "default"
queue = "default"
```

When running the worker for the first time (or sending a job), best
attempt is made to ensure the DB exists (if not, we will try to create
it) and the `pgmq` schema is initialized. This allows for hassle-free
maintenance and easier onboarding.

The way this is done is that we use the gargantext DB credentials as a
pivot to create the worker DB. This works of course if both DBs are on
the same server (`host` and `port` for both definitions must agree)
and the garg DB user can create new DBs. If they're on different
server, it is just assumed that the `worker.database` from above
config exists. See `G.C.W.Broker` -> `initBrokerWithDBCreate` for more
details.
## Steps to configure the worker

1. First create our Worker
```bash
createdb gargantext_pgmq
psql -c "ALTER DATABASE \"gargantext_pgmq\" OWNER TO \"gargantua\""
```
2. Then configure you gargantext-settings.toml (see next section)

3. Finally launch the worker
```bash
nix-shell --run "cabal v2-run gargantext -- worker run --name default"
```

Or launch all worker definitions at once:
```bash
nix-shell --run "cabal v2-run gargantext -- worker run-all"
```

## Configuration

Edit your `gargantext-settings.toml` file and add this section:
```toml
[worker]

# After this number of seconds, the job will be available again.
# You can set timeout for each job individually and this is the
# preferred method over using defaultVt.
default_visibility_timeout = 1

# default delay before job is visible to the worker
default_delay = 0

[worker.database]
host = "127.0.0.1"
port = 5432
name = "pgmq"
user = "gargantua"
pass = PASSWORD_TO_CHANGE

[[worker.definitions]]
name = "default"
queue = "default"
```

When running the worker for the first time (or sending a job), best
attempt is made to ensure the DB exists (if not, we will try to create
it) and the `pgmq` schema is initialized. This allows for hassle-free
maintenance and easier onboarding.

The way this is done is that we use the gargantext DB credentials as a
pivot to create the worker DB. This works of course if both DBs are on
the same server (`host` and `port` for both definitions must agree)
and the garg DB user can create new DBs. If they're on different
server, it is just assumed that the `worker.database` from above
config exists. See `G.C.W.Broker` -> `initBrokerWithDBCreate` for more
details.

### Notifications <a name="notifications"></a>

#### Configuration

See `gargantex-settings.toml_toModify`. Overall, the config for
notifications is quite simple:
```toml
[notifications]
central-exchange = { bind = "tcp://*:5560", connect = "tcp://localhost:5560" }
dispatcher = { bind = "tcp://*:5561", connect = "tcp://localhost:5561" }
```
The `bind`/`connect` strings are just forwarded to nanomsg, hence we
could switch `tcp` to `ipc` (but this limits us to a single machine)
or `inproc` (this limits us to inter-process communication).

The `bind` part is for the server, the `connect` part is for the
clients connecting to that server.

The notifications mechanism has 3 basic components: the `central
exchange`, the `dispatcher` and the `websocket server`.

## Configuration

See `gargantex-settings.toml_toModify`. Overall, the config for
notifications is quite simple:
```toml
[notifications]
central-exchange = { bind = "tcp://*:5560", connect = "tcp://localhost:5560" }
dispatcher = { bind = "tcp://*:5561", connect = "tcp://localhost:5561" }
```
The `bind`/`connect` strings are just forwarded to nanomsg, hence we
could switch `tcp` to `ipc` (but this limits us to a single machine)
or `inproc` (this limits us to inter-process communication).

The `bind` part is for the server, the `connect` part is for the
clients connecting to that server.
